
(* Arbres de syntaxe abstraite issus du parseur *)

type loc = Lexing.position * Lexing.position

type ident = { id_name : string; id_loc : loc }

type p_type =
  | PTboolean
  | PTinteger
  | PTreal
  | PTarray of loc * int * int * p_type
  | PTid of ident

type constant = 
  | Cboolean of bool
  | Cinteger of int
  | Creal of float

type unop = 
  | Unot | Uminus | Uplus 

type binop = 
  | Beq | Bneq | Blt | Ble | Bgt | Bge 
  | Badd | Bsub | Bmul | Bslash | Bdiv | Bmod
  | Band | Bor

type p_expr =
  { p_expr_node : p_expr_node;
    p_expr_loc : loc }

and p_expr_node =
  | PEconst of constant
  | PElvalue of p_lvalue
  | PEunop of unop * p_expr
  | PEbinop of binop * p_expr * p_expr
  | PEcall of ident * p_expr list

and p_lvalue =
  | PLident of ident
  | PLarrget of p_lvalue * p_expr

type p_pattern = { 
  p_pattern_node : p_pattern_node;
  p_pattern_loc : loc
}

and p_pattern_node = 
  | PPall
  | PPconst of constant
  | PPid of ident
  | PPconstructor of ident * p_pattern list

type p_statement = 
  { p_st_node : p_statement_node;
    p_st_loc : loc }

and p_statement_node =
  | PSskip
  | PSassign of p_lvalue * p_expr
  | PSprocedure_call of ident * p_expr list
  | PSif of p_expr * p_statement * p_statement
  | PSwhile of p_expr * p_statement
  | PSfor of ident * p_expr * p_expr * p_statement
  | PSbloc of p_statement list
  | PSmatch of p_expr * (p_pattern * p_statement list) list

type p_decl_vars = (ident * p_type) list

type p_constructor = ident * p_type list

type p_decl_types = (ident * p_constructor list) list

type mode = Value | Reference

type p_parameter = mode * ident * p_type

type p_definition_type = 
  | PDfunction of p_type
  | PDprocedure

type p_definition = 
  ident * p_parameter list * p_definition_type * p_decl_vars * p_statement

type p_file = { 
  p_name : ident;
  p_decl_types : p_decl_types;
  p_decl_vars : p_decl_vars;
  p_definitions : p_definition list;
  p_main : p_statement;
}

